/*******************************************************************************
*
* Freescale Semiconductor
* (c) Copyright Freescale Semiconductor 2004
* ALL RIGHTS RESERVED.
*
********************************************************************************
*
* File Name: fmaster.h
*
* Description: Header file for fmaster.c (FreeMaster communication)
*
* Modules Included: None
*
*******************************************************************************/

#ifndef _FMASTER_H_
#define _FMASTER_H_

#include "c_types.h"

/*.
    FREEMASTER Configuration - should be placed in appconfig.h
--------------------------------------------
.*/
#define FMASTER_INCLUDE_CMD_SCOPE 
#define FMASTER_INCLUDE_CMD_RECORDER
//#define FMASTER_INCLUDE_CMD_APPCMD
#define FMASTER_RECORDER_TIME_BASE       0x8100
#define FMASTER_REC_BUFF_LEN             0x0500
//#define FMASTER_APPCMD_BUFF_LEN          0x0f


/******************************************************************************/
/*       Free Master communication protocol version 3.0 specific part:        */
/*                                                                            */
/*                            !!! DO NOT MODIFY !!!                           */
/******************************************************************************/
/*-------------------------------------
  command message - standard commands
  -------------------------------------*/

#define FMASTER_CMD_READMEM          0x01 /* read block of memory */
#define FMASTER_CMD_READMEMEX        0x04 /* v3+ */
#define FMASTER_CMD_WRITEMEM         0x02 /* write block of memory */
#define FMASTER_CMD_WRITEMEMMASK     0x03 /* write block of memory with mask */
#define FMASTER_CMD_WRITEMEMEX       0x05 /* v3+ */
#define FMASTER_CMD_WRITEMEMMASKEX   0x06 /* v3+ */
#define FMASTER_CMD_SETUPSCOPE       0x08 /* setup scope */
#define FMASTER_CMD_SETUPSCOPEEX     0x0A /* v3+ */
#define FMASTER_CMD_SETUPREC         0x09 /* setup recorder */
#define FMASTER_CMD_SETUPRECEX       0x0B /* v3+ */
#define FMASTER_CMD_CALLAPPCMD       0x10 /* call user application command */

/*-------------------------------------
   command message - special commands
  -------------------------------------*/
/* no data part */
#define FMASTER_CMD_GETINFO       0xC0   /* get system information */
#define FMASTER_CMD_GETINFOBRIEF  0xC8   /* get brief system information */
#define FMASTER_CMD_STARTREC      0xC1   /* start recorder */
#define FMASTER_CMD_STOPREC       0xC2   /* stop recorder */
#define FMASTER_CMD_GETRECSTS     0xC3   /* get recorder status */
#define FMASTER_CMD_GETRECBUFF    0xC4   /* get recorder buffer information */
#define FMASTER_CMD_GETRECBUFFEX  0xC9   /* v3+ */
#define FMASTER_CMD_READSCOPE     0xC5   /* read scope variables */
#define FMASTER_CMD_GETAPPCMDSTS  0xC6   
/* 2 bytes data part */
#define FMASTER_CMD_READVAR8      0xD0   
#define FMASTER_CMD_READVAR16     0xD1   
#define FMASTER_CMD_READVAR32     0xD2   
/* 4 bytes data part */
#define FMASTER_CMD_WRITEVAR8     0xE3
#define FMASTER_CMD_WRITEVAR16    0xE4
#define FMASTER_CMD_READVAR8EX    0xE0   
#define FMASTER_CMD_READVAR16EX   0xE1   
#define FMASTER_CMD_READVAR32EX   0xE2   

/* 6 bytes data part */
/* write 32-bit variable */
#define FMASTER_CMD_WRITEVAR32    0xF0

/* get info command parameters */
#define FMASTER_PROT_VER          3       /* FreeMaster protocol version 3.0 */
#define FMASTER_DATABUSWDT        1       /* data bus width 8 bit */
#define FMASTER_CFG_FLAFGS        0x0001  /* big endian */
#define FMASTER_IDT_STRING_LEN    25      /* length of identification string */

/*-------------------------------------
  response message - status byte
  -------------------------------------*/
/* confirmation codes */
#define FMASTER_STC_OK            0x00   /* operation succesful */
#define FMASTER_STC_RECRUN        0x01   /* recorder running */
#define FMASTER_STC_RECDONE       0x02   /* recorder already finished */

/* error codes */
#define FMASTER_STC_INVCMD        0x81   /* invalid command */
#define FMASTER_STC_CMDSERR       0x82   /* checksum error */
#define FMASTER_STC_CMDBUFFOVF    0x83   /* command too long */
#define FMASTER_STC_RSPBUFFOVF    0x84   /* response would be too long */
#define FMASTER_STC_INVBUFF       0x85   /* invalid buffer length specified */
#define FMASTER_STC_INVSIZE       0x86   /* invalid size */
#define FMASTER_STC_SERVBUSY      0x87   /* service is busy */
#define FMASTER_STC_NOTINIT       0x88   /* scope/recorder not configured */
#define FMASTER_STC_UNKNOWN       0xFF   /* reserved */

/* recorder trigger modes */
#define FMASTER_REC_TRIGOFF       0      /* manual mode (trigger disabled) */
#define FMASTER_REC_TRIGRIS       1      /* rising edge */
#define FMASTER_REC_TRIGFAL       2      /* falling edge */

/* recorder trigger data formats */
#define TRG_UNSIGNED_CHAR           0x02
#define TRG_SIGNED_CHAR             0x03
#define TRG_UNSIGNED_INT            0x04
#define TRG_SIGNED_INT              0x05
#define TRG_UNSIGNED_DWORD          0x08
#define TRG_SIGNED_DWORD            0x09

/******************************************************************************/
/*                                                                            */
/*   FreeMaster buffer lengths and other default values:                      */
/*	                                                                          */
/*   FMASTER_BUFFER_SIZE         - FreeMaster input/output buffer size        */
/*   FMASTER_REC_BUFF_LEN        - FreeMaster Recorder buffer size            */
/*   FMASTER_APPCMD_BUFF_LEN     - FreeMaster Applciation Command buffer size */
/*   FMASTER_RECORDER_TIME_BASE  - FreeMaster Recorder time base value        */
/*   FMASTER_GLOB_VERSION_MAJOR  - board firmware major number                */
/*   FMASTER_GLOB_VERSION_MINOR  - board firmware minor number                */
/*   FMASTER_IDT_STRING          - device identification string               */
/*                                                                            */
/*   NOTE: This values should be defined in the appconfig.h file              */
/*                                                                            */
/*   E.G.                                                                     */
/*                                                                            */
/*   #define FMASTER_BUFFER_SIZE 150                                          */
/*   (this code placed in the appconfig.h file will enlarge the FreeMaster    */
/*   input/output buffer size to 150 from default value 40)                   */
/*	                                                                          */
/******************************************************************************/

#ifndef FMASTER_BUFFER_SIZE
  /* FreeMaster input/output buffer size */
  #define FMASTER_BUFFER_SIZE 40
#endif

#ifndef FMASTER_REC_BUFF_LEN
  /* recorder buffer length */
  #define FMASTER_REC_BUFF_LEN 200
#endif

#ifndef FMASTER_RECORDER_TIME_BASE
  /* recorder time base default value */
  #define FMASTER_RECORDER_TIME_BASE	0x8000
#endif

#ifndef FMASTER_APPCMD_BUFF_LEN
/* Application Command buffer length */
#define FMASTER_APPCMD_BUFF_LEN 10
#endif

#ifndef FMASTER_GLOB_VERSION_MAJOR
  /* board firmware version major number */
  #define FMASTER_GLOB_VERSION_MAJOR	0
#endif

#ifndef	FMASTER_GLOB_VERSION_MINOR
  /* board firmware version minor number */
  #define FMASTER_GLOB_VERSION_MINOR	0
#endif

#ifndef FMASTER_IDT_STRING
  /* device identification string */
  #define FMASTER_IDT_STRING "FreeMaster communication !"
#endif

#ifndef FMASTER_COMM_MODULE
    /* set FreeMaster software communication module */
    /* available modules:  ESCI_A
                           ESCI_B  */
#define FMASTER_COMM_MODULE ESCI_A

#endif

/******************************************************************************/
/*      GLOBAL FUNCTION DECLARATION                                           */
/******************************************************************************/

Word16 fmasterInit(void);
void iniFmasterUart(void);

#ifdef FMASTER_INCLUDE_CMD_APPCMD
  unsigned char fmasterGetAppCmdSts(void);
#else
  #define fmasterGetAppCmdSts() 0 /* no new command received */
#endif

#ifdef FMASTER_INCLUDE_CMD_APPCMD
  char fmasterWriteAppCmdSts(unsigned char state);
#else
  #define fmasterWriteAppCmdSts(x) 0
#endif

#ifdef FMASTER_INCLUDE_CMD_RECORDER
  void fmasterRecorder(void);
#else
  #define fmasterRecorder()
#endif

void fmasterDispatcher(void);


/********************************************************************/
/* FREEMASTER UART CHANNEL MAPPING ADDED FOR PUMA                   */
/********************************************************************/
#define FMASTER_UMR MCF_UART_UMR0
#define FMASTER_USR MCF_UART_USR0
#define FMASTER_UCR MCF_UART_UCR0
#define FMASTER_UTB MCF_UART_UTB0
#define FMASTER_URB MCF_UART_URB0
#define FMASTER_UISR MCF_UART_UISR0
#define FMASTER_UIMR MCF_UART_UIMR0
#define FMASTER_UCSR MCF_UART_UCSR0
#define FMASTER_UBG1 MCF_UART_UBG10
#define FMASTER_UBG2 MCF_UART_UBG20


#define FMASTER_GPIO_INIT 0x0F
#define MCF_UART_UIMR_NOINT 0

#define periphBitSet(Mask, Addr)        *(Addr) |= Mask
#define periphBitClear(Mask, Addr)      *(Addr) &= ~(Mask)

#define FMASTER_BAUDRATE	19200
#define FMASTER_BAUDRATE_INI1 ((((SYSTEM_CLOCK*1000000)/(FMASTER_BAUDRATE * 32)) >> 8) & 0x00FF)
#define FMASTER_BAUDRATE_INI2 (((SYSTEM_CLOCK*1000000)/(FMASTER_BAUDRATE * 32)) & 0x00FF)


#endif

/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
